{%- if 'singlehtml' not in builder %}
<div role="search">
    <form id="rtd-search-form" class="wy-form" action="{{ pathto('search') }}" method="get">
        <input type="text" name="q" placeholder="{{ _('Search docs') }}" style="height: 33px; width: 175px; border-radius: 0.25rem; border: 0;" />
        <input type="hidden" name="check_keywords" value="yes" />
        <input type="hidden" name="area" value="default" />
        <button type="submit" class="btn btn-light" style="height: 33px">
            <i class="fa fa-search"></i>
        </button>
    </form>
</div>

<style>
    .sidebar-collapse form#rtd-search-form {
        display: none;
    }

    .sidebar-collapse .main-sidebar:hover form#rtd-search-form {
        display: block;
    }
</style>
{%- endif %}
