<script type="text/javascript">
    var DOCUMENTATION_OPTIONS = {
        URL_ROOT: '{{ url_root }}',
        VERSION: '{{ release|e }}',
        LANGUAGE: '{{ language }}',
        COLLAPSE_INDEX: false,
        FILE_SUFFIX: '{{ '' if no_search_suffix else file_suffix }}',
        HAS_SOURCE:  {{ has_source|lower }},
        SOURCELINK_SUFFIX: '{{ sourcelink_suffix }}',
    };
</script>