{%- block doctype -%}{%- if html5_doctype %}
<!DOCTYPE html>
{%- else %}
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
{%- endif %}{%- endblock %}
{%- set url_root = pathto('', 1) %}
{# XXX necessary? #}
{%- if url_root == '#' %}{% set url_root = '' %}{% endif %}
<html>
{%- macro css() %}
    <link rel="stylesheet" href="{{ pathto('_static/' + style, 1) }}" type="text/css" />
    {%- for css in css_files %}
      {%- if css|attr("rel") %}
    <link rel="{{ css.rel }}" href="{{ pathto(css.filename, 1) }}" type="text/css"{% if css.title is not none %} title="{{ css.title }}"{% endif %} />
      {%- else %}
    <link rel="stylesheet" href="{{ pathto(css, 1) }}" type="text/css" />
      {%- endif %}
    {%- endfor %}
    {% if theme_canonical_url %}
    <link rel="canonical" href="{{ theme_canonical_url }}{{ pagename }}.html"/>
    {% endif %}
    <link rel="stylesheet" href="{{ pathto('_static/basic.css', 1) }}" type="text/css" />
{%- endmacro %}
{%- macro script() %}
    <script>
        window.hostname = window.location.origin || window.location.protocol + '//' + window.location.host;
    </script>
    <script type="text/javascript" id="documentation_options" data-url_root="{{ url_root }}"
            src="{{ pathto('_static/documentation_options.js', 1) }}"></script>
    {%- for scriptfile in script_files|unique %}
        <!-- 1 links -->
        {{ js_tag(scriptfile) }}
    {%- endfor %}
    <script type="text/javascript" src="{{ pathto('_static/js/theme.js', 1) }}"></script>
{%- endmacro %}
<head>
    <meta http-equiv="Content-Type" content="text/html; charset=utf-8"/>
    <meta name="robots" content="NONE,NOARCHIVE" />
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta content="width=device-width, initial-scale=1, maximum-scale=1, user-scalable=no" name="viewport">
    <title>{{ project }} documentation</title>
    {%- block css_basic %}
        {{- css() }}
    {%- endblock %}
    <script type="text/javascript" src="{{ pathto('_static/bundle/doc.js', 1) }}"></script>
    {%- block css %}
    {%- endblock %}
    <link rel="shortcut icon" href="{{ pathto('_static/img/logo/favicon.ico', 1) }}" />
    {%- block scripts %}
        {{- script() }}
    {%- endblock %}
{%- block extrahead %} {% endblock %}
</head>

<body class="hold-transition gui-skin-default sidebar-mini">
<div id="loader-wrapper">
    <div id="loader"></div>
</div>
<div class="wrapper" >
    {% if theme_github_banner|lower != 'false' %}
    <a href="https://github.com/{{ theme_github_user }}/{{ theme_github_repo }}" class="fork-me">
        <img src="https://s3.amazonaws.com/github/ribbons/forkme_right_darkblue_121621.png" alt="Fork me on GitHub" />
    </a>
    {% endif %}
    <header class="main-header">
        <!-- Header Navbar: style can be found in header.less -->
        <nav class="navbar navbar-static-top">
            <!-- Sidebar toggle button-->
            <a class="nav-link" data-widget="pushmenu" onclick="saveHideMenuSettings()" href="#"><i class="fa fa-bars  ico-data-default"></i></a>
        </nav>
    </header>
    <!-- Left side column. contains the logo and sidebar -->
    <aside class="main-sidebar sidebar-dark-primary elevation-4">
        <!-- Logo  -->
        {% block logo %}
            <!-- Logo -->
            <a href="index.html" alt="{{ project_gui_name|default(project) }}" id="root_link" class="brand-link logo app-logo">
                <span class="brand-text font-weight-light">{{ project_gui_name|default(project) }}</span>
            </a>
        {% endblock %}
        <!-- sidebar: style can be found in sidebar.less -->
        <div class="sidebar">
            <!-- sidebar menu: : style can be found in sidebar.less -->

        <nav class="mt-2">
            <ul class="sidebar-menu nav nav-pills nav-sidebar flex-column ico-menu" data-widget="treeview" role="menu" data-accordion="false">
                <li class="nav-header">DOCS NAVIGATION</li>
                {{ toctree(includehidden=true, collapse=true) }}
            </ul>
            <!-- github link  -->
            {% if theme_github_user and theme_github_repo %}
                {% if theme_github_button|lower == 'true' %}
                <h3>Watch us on GitHub</h3>
                <p>
                <iframe src="https://ghbtns.com/github-btn.html?user={{ theme_github_user }}&repo={{ theme_github_repo }}&type={{ theme_github_type }}&count={{ theme_github_count }}&size=large&v=2"
                  allowtransparency="true" frameborder="0" scrolling="0" width="200px" height="35px"></iframe>
                </p>
                {% endif %}
            {% endif %}
            <!-- quick search -->
            {%- block sidebarsearch %}
                {%- include "searchbox.html" %}
            {%- endblock %}
        </nav>
        </div>
        <!-- /.sidebar -->
    </aside>

    <!-- Content Wrapper. Contains page content -->
    <div class="content-wrapper" id='spajs-right-area'>
        <!--  Main content -->

        <!-- For notification about not up-to-date-version -->
        <div class="body" role="main"></div>
        <!-- -->

        <section class="content-header">
                <h1>{{ project }} documentation</h1>
        </section>
        <section class="content">
            <div class="container-fluid">
                <div class="card card-info">
                    <div class="card-body">
                        <div class="body">
                            {%- block content %}
                                {%- block document %}
                                     {% block body %} {% endblock %}
                                {%- endblock %}
                            {%- endblock %}
                        </div>
                    </div>
                </div>
            </div>
        </section>
    </div>

    {% block footer %}
    <footer class="main-footer">
        {% block additional_information %}
        {% endblock %}
         <div class="pull-right d-none d-sm-block">
             &copy; {{ copyright }}
         </div>
        <br>
    </footer>
    {% endblock %}
</div>
    {% include "rtd-versions.html" %}
</body>

</html>
