# Generated by Django 2.2.18 on 2021-03-02 08:27

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0011_update_proxy_permissions'),
        ('vstutils_api', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='TwoFactor',
            fields=[
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, primary_key=True, related_name='twofa', serialize=False, to=settings.AUTH_USER_MODEL)),
                ('secret', models.CharField(max_length=256)),
            ],
            options={
                'default_related_name': 'twofa',
            },
        ),
        migrations.CreateModel(
            name='RecoveryCode',
            fields=[
                ('id', models.AutoField(max_length=100, primary_key=True, serialize=False)),
                ('key', models.CharField(max_length=256)),
                ('tfa', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='recoverycode', to='vstutils_api.TwoFactor')),
            ],
            options={
                'default_related_name': 'recoverycode',
            },
        ),
    ]
