# flake8: noqa: E501
TRANSLATION = {
    'new': 'Mới',
    'Create': 'Tạo',
    'add': 'thêm',
    'Edit': 'Chỉnh sửa',
    'edit': 'Chỉnh sửa',
    'Save': 'lưu',
    'Remove': 'Xóa',
    'Reload': 'Làm mới',
    'send': 'gửi',
    'select': 'chọn',
    'filters': 'bộ lọc',
    'apply': 'áp dụng',
    'cancel': 'Hủy',
    'field': 'tường',
    'actions': 'thao tác',
    'sublinks': 'đường dẫn',
    'name': 'tên',
    'type': 'loại',
    'owner': 'chủ sở hữu',
    'notes': 'ghi chú',
    'copy': 'sao chép',
    'error': 'lỗi',
    'home': 'trang chủ',
    'login': 'đăng nhập',
    'logout': 'đăng xuất',
    'Profile': 'Hồ sơ',
    'settings': 'thiết lập',
    'Settings': 'Thiết lập',
    'version': 'phiên bản',
    'cache': 'bộ nhớ đệm',
    'documentation': 'tài liệu',
    'request': 'yêu cầu',
    'repository': 'kho thông tin',
    'app info': 'thông tin',
    'status': 'trạng thái',
    'List is empty': 'Danh sách trống',
    'child instances': 'thêm các mục lồng',
    'enter value': 'nhập giá trị',
    'more info': 'thêm thông tin',
    'search by': 'tìm kiếm theo',
    'Search': 'Tìm kiếm ',
    'Selected': 'Đã chọn',
    'Nothing found!': 'Không tìm thấy gì!',
    'Main settings': 'cài đặt chính',
    'Custom settings': 'Cài đặt tùy chỉnh',
    'Remove from list': 'Xóa từ danh sách',
    'Purge': 'Xóa',
    'Select an action for this instances': 'Chọn một hành động cho trường hợp này',
    'Username not': 'Tên người dùng không phải',

    # fields
    'Id': 'định danh',
    'ID': 'định danh',

    # x-menu generic
    'system': 'Hệ thống',
    'users': 'Người dùng',
    'Is active': 'Hoạt động',
    'Is staff': 'Người điều hành',

    # multiactions button
    'actions on': 'thao tác trên',
    'on item': 'thành phần',

    'key': 'Chìa khóa',
    'value': 'Giá trị',
    'new owner': 'Chủ sở hữu mới',
    'detail': 'Chi tiết',
    'description': 'Miêu tả',
    'username': 'Tên người dùng',
    'is active': 'Hoạt động',
    'user': 'Người dùng',
    'yes': 'Có',
    'no': 'Không',
    'is staff': 'Admin',
    'first name': 'Tên',
    'last name': 'Họ',
    'password': 'Mật khẩu',
    'Password': 'Mật khẩu',
    'Change password': 'Đổi mật khẩu',
    'Change_password': 'Đổi mật khẩu',
    'generate password': 'Viết mật khẩu',
    'repeat password': 'Lập lại mật khẩu',
    'Repeat password': 'Lập lại mật khẩu',
    'Old password': 'Mật khẩu cũ',
    'New password': 'Mật khẩu mới',
    'Confirm new password': 'Xác nhận mật khẩu mới',
    'kind': 'Loại',
    'mode': 'Chế độ',
    'options': 'Tùy chọn',
    'information': 'Thông tin',
    'date': 'Ngày',

    'file n selected': 'File chưa được chọn',
    'image n selected': 'Hình ảnh chưa được chọn',

    # filters
    'Ordering': 'Thứ tự phân loại',
    'is_active': 'Hoạt động',
    'Username': 'Tên đăng nhập',
    'First name': 'Tên',
    'Last name': 'Họ',

    'Page matching current url was not found': 'Không tìm thấy url hiện tại phù hợp với trang',
    'Unknown error': 'Lỗi không thể nhận biết',
    'Error': 'Lỗi',

    'We\'ve emailed you instructions for setting your password, if an account exists with the email you entered. You should receive them shortly.': 'Hướng dẫn hủy mật khẩu đã được gửi vào e-mail của bạn.',
    'If you don\'t receive an email, please make sure you\'ve entered the address you registered with, and check your spam folder.': '"Nếu bạn chưa nhận được thư, hãy kiểm tra lại địa chỉ e-mail và mục Spam."',
    'The password reset link was invalid, possibly because it has already been used. Please request a new Password reset.': '"Đường dẫn không đúng, có thể đã được sử dụng. Xin hãy thử lại lần nữa."',
    'Please enter your new password twice so we can verify you typed it in correctly.': 'Nhập mật khẩu mới',
    'change my password': 'Thay đổi mật khẩu',
    'Your password has been set.  You may go ahead and log in now.': 'Thay đổi mật khẩu thành công. Hãy đăng nhập.',
    'Please enter a correct %(username)s and password. Note that both fields may be case-sensitive.': 'Vui lòng nhập chính xác% (tên người dùng) s và mật khẩu',
    'or': 'hoặc',
    'two-factor authentication code': 'hai bước xác thực (2FA)',
    'verify code': 'xác nhận mã',
    'Enter the code from the two-factor app on your mobile device. If you\'ve lost your device, you may enter one of your recovery codes.': '"Nhập mã từ ứng dụng sử dụng hai bước xác thực trên thiết bị di động của bạn.Nếu bạn mất thiết bị của mình, bạn có thể nhập một trong các mã khôi phục."',
    'Scan the image with the two-factor authentication app on your phone. If you can’t use a barcode, enter this text code instead.': '"Quét hình ảnh bằng ứng dụng xác thực hai bước trên điện thoại của bạn. Nếu bạn không thể quét hình ảnh, hãy nhập mã văn bản."',
    'Enter the six-digit code from the application': 'Điền mã xác thực từ ứng dụng',
    'code': 'Mã',
    'recovery codes': 'Mã khôi phục',
    'Recovery codes are used to access your account in the event you cannot receive two-factor authentication codes.': 'Mã khôi phục được sử dụng để truy cập vào tài khoản của bạn nếu bạn không thể nhận được mã xác thực hai bước.',
    'Disabling Two-Factor Authentication (2FA) will decrease your account\'s security': 'mã xác thực hai bước.',
    'Enable': 'Bật',
    'Disable': 'Tắt',
    'Two factor authentication': 'Xác thực hai bước (2FA)',
    'No {0} matches the given query.': 'Không tìm thấy',
    'Password is not correct.': 'Sai mật khẩu',
    'New passwords values are not equal.': 'Mật khẩu mới không chính xác',
    'This password is entirely numeric.': 'Mật khẩu này chỉ có chữ số.',
    'Passwords do not match.': 'Mật khẩu không phù hợp.',
    "The two password fields didn't match.": 'Hai trường mật khẩu không khớp với nhau.',
    'Dark theme': 'Màn hình tối',
    'Language': 'Ngôn ngữ',
    'height': 'chiều cao',
    'width': 'chiều rộng',
    'More info': 'thêm thông tin',
    'Dark mode': 'Chủ đề tối',
    'Execute': 'Hành hình',
    'Domain should be setuped or username should be with @domain.name': "Tên miền phải được thiết lập hoặc tên người dùng phải bằng @ domain.name",
    'Invalid name': 'Tên không hợp lệ',
    'Should be [full.domain.name]': 'Phải là [full.domain.name].',
    'Invalid registration email send.': 'Email đăng ký không hợp lệ gửi.',
    'To continue, need to accept the terms agreement.': 'Để tiếp tục, cần chấp nhận thỏa thuận điều khoản.',

    # Проверить
    'Changes in settings are successfully saved. Please refresh the page.': 'Các thay đổi trong cài đặt đã được lưu thành công. Vui lòng làm mới trang.',
    'now': 'hiện nay',
    'later': 'một lát sau',
    'I accept the ': 'Tôi chấp nhận ',
    'Zoom': 'Thu phóng',
    'Scale': 'Tỉ lệ',
    'This file format is not supported': 'Định dạng tệp này không được hỗ trợ',
    'Oops! Something went wrong.': 'Một cái gì đó đã đi sai.',
    '[Object not found]': '[Không tìm thấy đối tượng]',
    'Execute action on {0}': 'Thực hiện hành động trên {0}',
    '{n} instance': '{n} trường hợp',
    'Open image': 'Mở hình ảnh',
    'Confirm action': 'Xác nhận hành động',
    'Confirm': 'Xác nhận',
    'Cancel': 'Hủy bỏ',

    # field's validation
    # empty
    'Field "<b>{0}</b>" is empty.': 'Trường "<b> {0} </b>" trống không',
    # required
    'Field "<b>{0}</b>" is required.': 'Trường "<b> {0} </b>"  bắt buộc.',
    # minLength
    'Field "<b>{0}</b>" is too short.<br> Field length should not be shorter, then {1}.': 'Độ dài của trường "<b> {0} </b>" quá ngắn. <br> Độ dài của trường không được nhỏ hơn {1}.',
    # maxLength
    'Field "<b>{0}</b>" is too long. <br> Field length should not be longer, then {1}.': 'Độ dài của trường "<b> {0} </b>" quá dài. <br> Trường không được dài hơn {1}.',
    # min
    'Field "<b>{0}</b>" is too small.<br> Field should not be smaller, then {1}.': 'Giá trị của trường "<b> {0} </b>" quá nhỏ. <br> Giá trị của trường không được nhỏ hơn {1}.',
    # max
    'Field "<b>{0}</b>" is too big.<br> Field should not be bigger, then {1}.': 'Giá trị của trường "<b> {0} </b>" quá lớn. <br> Giá trị của trường không được lớn hơn {1}.',
    # invalid
    '<b>{0} </b> value is not valid for <b>{1}</b> field.': 'Giá trị <b> {0} </b> không hợp lệ cho trường <b> {1} </b>.',
    # email
    '<b>"{0}"</b> field should be written in <b>"example@mail.com"</b> format.': 'Trường <b> "{0}" </b> phải làm như <b> "example@mail.com" </b>.',

    # validators
    'imageValidationResolutionError': 'Phải có ít nhất {min} và không nhiều hơn {max} pixel.',

    # instance operation success
    # add
    'Child "<b>{0}</b>" instance was successfully added to parent list.': 'Đối tượng con "<b> {0} </b>" đã được thêm thành công vào danh sách mẹ.',
    # create
    'New "<b>{0}</b>" instance was successfully created.': 'Đối tượng mới "<b> {0} </b>" đã được tạo thành công.',
    # remove
    '"<b>{0}</b>" {1} was successfully removed.': '"<b>{0}</b>" {1} đã được xóa thành công."',
    # save
    'Changes in "<b>{0}</b>" {1} were successfully saved.': 'Thay đổi đối tượng {1} "<b> {0} </ b>" đã được lưu thành công.',
    # execute
    'Action "<b>{0}</b>" was successfully executed on "<b>{1}</b>" instance.': 'Thao tác "<b> {0} </b>" đã được thực hiện thành công trên đối tượng "<b> {1} </b>".',
    # instance operation error
    # add:
    'An error occurred during adding of child "<b>{0}</b>" instance to parent list.<br> Error details: {1}': 'Khi thêm đối tượng con "<b> {0} </b>" vào danh sách mẹ đã xảy ra lỗi <br> thêm chi tiết: {1}',
    # create:
    'An error occurred during creation.<br>Error details:<br>{0}': 'Đã xảy ra một số lỗi khi tạo phiên bản mới. <br> Chi tiết lỗi: <br> {0}',
    # remove:
    'An error occurred during removal process of "<b>{0}</b>" {1}.<br> Error details: {2}': 'Khi xóa {1} "<b> {0} </b>". đã xảy ra lỗi <br> thêm chi tiết: {2}',
    # save:
    'An error occurred during process.<br>Error details:<br>{0}': 'Đã xảy ra một số lỗi trong quá trình lưu. <br> Chi tiết lỗi: <br> {0}',
    # execute:
    'An error occurred during <b>{0}</b>. Error details:<br>{1}': 'Đã xảy ra một số lỗi trong <b> {0} </b>. Chi tiết lỗi: <br> {1}',
    'Link': 'Liên kết',

    # csv
    'Actions': 'Hành động',
    'Index': 'Mục lục',
    'Download with headers': 'Tải xuống với tiêu đề',
}

SERVER_TRANSLATION = {
    # field's validation
    "value is not a valid list": "giá trị không phải là một danh sách hợp lệ",
    "value is not a valid JSON": "giá trị không phải là JSON hợp lệ",
    "key {missing_key} is missing": "khóa {missing_key} bị thiếu",
    "invalid key {invalid_key}": "khóa không hợp lệ {invalid_key}",

    # field help_text
    'A confirmation will be sent to your e-mail': 'Một xác nhận sẽ được gửi đến e-mail của bạn',
    'Required. Inform a valid email address': 'Yêu cầu. Thông báo một địa chỉ email hợp lệ',

    # validators
    'unsupported image file format, expected ({0}), got {1}': 'Định dạng hình ảnh không được hỗ trợ, Được hỗ trợ ({0}), đã nhận {1}',
    'For some reason, this image file cannot be opened': 'Không thể mở file hình ảnh',
    'Invalid image {0}. Expected from {1} to {2}, got {3}': 'Hình ảnh {0} không chính xác. Dự kiến ​​từ {1} đến {2}, đã nhận {3}',
    'This field must contain only digits but input: ': 'Trường này chỉ được chứa các chữ số nhưng nhập: ',
    'Unsupported image file format': 'Định dạng tệp hình ảnh không được hỗ trợ',
    'is not in listed supported types': 'không có trong các loại được hỗ trợ được liệt kê',
    'Invalid image size orientations': 'Kích thước hình ảnh không đúng',
    'Current image size': 'Kích thước hình ảnh hiện tại',

    # agreements
    'personal data processing policy': 'Chính sách xử lý dữ liệu cá nhân',
    'I agree with ': 'Tôi đồng ý với ',
    'the personal data processing policy': 'chính sách xử lý dữ liệu cá nhân',
    'To continue, need to agree to the personal data processing policy.': 'Để tiếp tục, cần đồng ý với chính '
                                                                          'sách xử lý dữ liệu cá nhân.',
    'terms of agreement': 'các điều khoản của thỏa thuận',

    # auth/reg
    'sign in to start your session': 'đăng nhập để bắt đầu',
    'confirm your email before logging in': 'xác nhận email của bạn trước khi đăng nhập',
    'Confirmation link is invalid or expired': 'Liên kết xác nhận không hợp lệ hoặc hết hạn',
    'sign in': 'đăng nhập',
    'sign up': 'đăng ký',
    'forgot password': 'quên mật khẩu',
    'confirm password': 'xác nhận mật khẩu',
    'register': 'đăng ký',
    'fields with * is required': 'các mục có dấu * là bắt buộc',
    'Forgot your password? Enter your email address below, and an email with instructions for setting a new one will be sent.': 'Quên mật khẩu? Nhập E-mail để nhận hướng dẫn khôi phục mật khẩu',
    'reset my Password': 'đặt lại mật khẩu của tôi',
    'Invalid authentication code': 'Mã xác thực không hợp lệ',
    'Secret string must be provided': 'Chuỗi bí mật phải được cung cấp',

    # filters help text
    'A unique integer value (or comma separated list) identifying this instance.': 'Giá trị số duy nhất',
    'A name string value (or comma separated list) of instance.': 'Tên đối tượng',
    'Which field to use when ordering the results.': 'Trường để sắp xếp kết quả',
    "Search by {}'s primary key or {}": 'Tìm kiếm theo khóa chính của {} hoặc {}',
    '_name': 'tên',
    'Boolean value meaning status of user.': 'Giá trị boolean cho biết trạng thái của người dùng',
    'Users first name.': 'Tên người dùng',
    'Users last name.': 'Họ người dùng',
    'Users e-mail value.': 'E-mail người dùng',
}
