"use strict";(self.webpackChunk_vstconsulting_vstutils=self.webpackChunk_vstconsulting_vstutils||[]).push([[463],{40463:function(t,s,n){n.r(s),n.d(s,{bootstrap:function(){return u}});var u=n(43734);n(27632),n(12024),window.bootstrap=u}}]);