# Generated by Django 3.2.3 on 2021-08-13 05:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('vstutils_api', '0005_db_translations'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='usersettings',
            options={'default_related_name': '_settings'},
        ),
        migrations.RemoveField(
            model_name='usersettings',
            name='hidden',
        ),
        migrations.AlterField(
            model_name='usersettings',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
    ]
