{% extends "registration/base.html" %}

{% block login_body_content %}
    <p style="color: #000000">{% translate_text "We've emailed you instructions for setting your password, if an account exists with the email you entered. You should receive them shortly." %}</p>
    <p style="color: #000000">{% translate_text "If you don't receive an email, please make sure you've entered the address you registered with, and check your spam folder." %}</p>
{% endblock %}