{% extends "base.html" %}
{% load request_static %}
{% load cache %}

{% block gui_init_scripts %}
    <script type="text/javascript">
        // {% block gui_init_vars %}
        window.isDebug = {{ debug|yesno:"true,false" }};

        window.host_url = "{{ host_url }}";
        window.endpoint_url = "{{ host_url }}{{ endpoint_path }}";
        window.openapi_path = window.endpoint_url;

        window.project_gui_name = "{{ project_gui_name }}";

        window.project_version = "{{ project_version }}";
        window.gui_version = "{{ gui_version }}";
        window.gui_user_version = "{{ gui_user_version }}";

        window.is_superuser = {{ user.is_superuser|yesno:"true,false" }};
        window.is_staff = {{ user.is_staff|yesno:"true,false" }};

        // {% endblock %}
        // {% block gui_static_list %}
        window.resourceList = [
            // {% for file in static_files_list %}
                { priority: Number('{{ file.priority }}'), type: '{{ file.type }}', name: '{% static file.name %}?v={{file.version}}'},
            // {% endfor %}
        ];
        // {% endblock %}
    </script>
{% endblock %}

{% block body %}
    <body class="sidebar-mini {% block bodyclass %}{% endblock %}">
        <div id="RealBody"></div>

        {% block script %}{% endblock %}

        {% block app_loader %}
            <script src="{% static 'bundle/app_loader.js' %}?v={{ vstutils_version }}"></script>
        {% endblock %}
    </body>
{% endblock %}