{% load request_static %}

<!DOCTYPE html>
<html lang="en">
<head>
    <meta http-equiv="Content-Type" content="text/html; charset=utf-8"/>
    <meta name="robots" content="NONE,NOARCHIVE" />
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1, maximum-scale=2">
    {% block title %}<title>{% if name %}{{ name }} – {% endif %}{{ project_gui_name }}</title>{% endblock %}

    {% block favicon %}
        <link rel="shortcut icon" href="{% static_path '/img/logo/favicon.ico' %}">
    {% endblock %}

    <style>
        body {
            background: #ecf0f5;
            margin: 0;
            font-family: "Source Sans Pro",-apple-system,BlinkMacSystemFont,"Segoe UI",Roboto,"Helvetica Neue",Arial,sans-serif,"Apple Color Emoji","Segoe UI Emoji","Segoe UI Symbol";
            font-size: 1rem;
            font-weight: 400;
            line-height: 1.5;
            color: #212529;
            text-align: left;
        }

        .wrapper {
            margin-top: 10%;
        }

        .center {
            text-align: center;
        }
    </style>
</head>
<body>
<div class="wrapper">
    {% block logo_or_name %}
        <h1 class="center"> {{ project_gui_name }}</h1>
    {% endblock %}
    <h2 class="center">Oops, it looks like your connection with {{ project_gui_name }} server has been lost!</h2>
    <h3 class="center">Please, check out network connectivity and working status of your {{ project_gui_name }} server.</h3>
    <h3 class="center">Try to fix connection and continue working!</h3>
    <h3 class="center">{{ project_gui_name }} is waiting for you!</h3>
    <h3 class="center">Have a good day :)</h3>
</div>
</body>
</html>
