{% extends "rest_framework/base.html" %}

{% block title %}{{ project_gui_name }}{% endblock %}

{% block branding %}
    <a class='navbar-brand' rel="nofollow" href='/'>
        {{ project_gui_name }}
    </a>
{% endblock %}