.content-wrapper>section.content{padding:0}.card-body{padding:1rem}:root{--toclevel-margin:5px}.toctree-l1{margin-left:calc(var(--toclevel-margin) * 1)}.toctree-l2{margin-left:calc(var(--toclevel-margin) * 2)}.toctree-l3{margin-left:calc(var(--toclevel-margin) * 3)}.toctree-l4{margin-left:calc(var(--toclevel-margin) * 4)}.admonition{padding:10px}.section>h2{text-decoration:underline}