{% extends "registration/base.html" %}

{% block login_body_content %}
    {% if validlink %}
        <p>{% translate_text "Please enter your new password twice so we can verify you typed it in correctly." %}</p>
        <form method="post">
            {% csrf_token %}
            <fieldset class="module aligned">
                <div style="margin-bottom: 25px">
                    <div style="margin: 0 auto 25px auto;width: min-content;clear:both; text-align:right; line-height:25px;">
                        {{ form.new_password1.errors }}
                        <label for="id_new_password1" class="password-reset-label">{% translate_text "new password" capfirst %}:</label>
                        {{ form.new_password1 }}
                    </div>
                    <div style="margin: 0 auto 25px auto;width: min-content;clear:both; text-align:right; line-height:25px;">
                        {{ form.new_password2.errors }}
                        <label for="id_new_password2" class="password-reset-label">{% translate_text "confirm password" title %}:</label>
                        {{ form.new_password2 }}
                    </div>
                </div>
                <div style="text-align: center">
                    <input type="submit" value="{% translate_text 'change my password' capfirst %}" class="btn btn-primary">
                </div>
            </fieldset>
        </form>
    {% else %}
        <p>{% translate_text "The password reset link was invalid, possibly because it has already been used. Please request a new Password reset." %}</p>
    {% endif %}
{% endblock %}