TRANSLATION = {
    'version': 'versions | version | versions',
    'app info': 'about',
    'on item': 'items | item | items',
    'file n selected': 'no files | 1 file selected | {n} files selected',
    'image n selected': 'no images | 1 image selected | {n} images selected',
    'imageValidationResolutionError': 'Must be between {min} and {max} px.',
    'instance': 'instance | instances',
    '_name': 'name',
}
