{% extends "registration/base.html" %}

{% block login_body_content %}
    <p style="color: #000000">{% translate_text "Forgot your password? Enter your email address below, and an email with instructions for setting a new one will be sent." %}</p>

    <form method="post">
        {% csrf_token %}
        <fieldset class="module aligned">
            <div style="margin: 0 auto 25px auto;width: min-content;">
                {{ form.email.errors }}
                <label for="id_email" style="color: #000000; margin-right: 10px">Email:</label>
                {{ form.email }}
            </div>
            <div style="text-align: center">
                <input type="submit" value='{% translate_text "reset my Password" capfirst %}' class="btn btn-primary">
            </div>
        </fieldset>
    </form>
{% endblock %}