"use strict";(self.webpackChunk_vstconsulting_vstutils=self.webpackChunk_vstconsulting_vstutils||[]).push([[281],{26281:function(u,e,n){n.r(e),n.d(e,{PortalVue:function(){return l.ZP},Vue:function(){return t.default},VueI18n:function(){return r.Z},VueRouter:function(){return i.Z},Vuex:function(){return o.ZP}});var t=n(70538),i=n(78345),o=n(20629),r=n(17152),l=n(72433);window.Vue=t.default,window.VueRouter=i.Z,t.default.use(i.Z),window.Vuex=o.ZP,t.default.use(o.ZP),window.VueI18n=r.Z,t.default.use(r.Z),window.PortalVue=l.ZP,t.default.use(l.ZP),t.default.directive("element-bound",{bind:function(u,e){(0,e.value)(u)}})}}]);