{% extends "registration/base.html" %}

{% block login_body_content %}
    <style>
        .login-box {
            width: auto;
            max-width: 900px;
        }
    </style>

    {% block header %}
        <h3 style="text-align: center">{% translate_text "$header_message" capfirst %}</h3>
    {% endblock %}

    {% block content %}
        <div class="agreement-area">
            {{ terms | safe }}
        </div>
    {% endblock %}

{% endblock %}