/*! For license information please see 683.chunk.js.LICENSE.txt */
"use strict";(self.webpackChunk_vstconsulting_vstutils=self.webpackChunk_vstconsulting_vstutils||[]).push([[683],{26683:function(e,t,o){o.r(t),o(2577).default.setLanguage(["ru","ru-RU"],{Markdown:"Markdown",WYSIWYG:"WYSIWYG",Write:"Редактор",Preview:"Просмотр",Headings:"Заголовки",Paragraph:"Абзац",Bold:"Жирный",Italic:"Курсив",Strike:"Зачеркнутый",Code:"Код",Line:"Линия",Blockquote:"Цитата","Unordered list":"Неупорядоченный список","Ordered list":"Упорядоченный список",Task:"Галочка",Indent:"Увеличить отступ",Outdent:"Уменьшить отступ","Insert link":"Вставить ссылку","Insert CodeBlock":"Вставить блок кода","Insert table":"Вставить таблицу","Insert image":"Вставить изображение",Heading:"Заголовок","Image URL":"URL изображения","Select image file":"Выбрать файл изображения","Choose a file":"Выбрать","No file":"Нет файла",Description:"Описание",OK:"Хорошо",More:"Еще",Cancel:"Отмена",File:"Файл",URL:"URL","Link text":"Текст ссылки","Add row to up":"Добавить строку вверх","Add row to down":"Добавить строку вниз","Add column to left":"Добавить столбец слева","Add column to right":"Добавить столбец справа","Remove row":"Удалить ряд","Remove column":"Удалить столбец","Align column to left":"Выровнять по левому краю","Align column to center":"Выровнять по центру","Align column to right":"Выровнять по правому краю","Remove table":"Удалить таблицу","Would you like to paste as table?":"Вы хотите вставить в виде таблицы?","Text color":"Цвет текста","Auto scroll enabled":"Автопрокрутка включена","Auto scroll disabled":"Автопрокрутка отключена","Choose language":"Выбрать язык"})}}]);