# flake8: noqa: E501
TRANSLATION = {
    'new': 'Новый',
    'Create': 'Создать',
    'add': 'добавить',
    'Edit': 'Редактировать',
    'edit': 'Редактировать',
    'Save': 'Сохранить',
    'Remove': 'Удалить',
    'Reload': 'Перезагрузить',
    'send': 'отправить',
    'select': 'выбрать',
    'filters': 'фильтры',
    'apply': 'применить',
    'cancel': 'отменить',
    'field': 'поле',
    'actions': 'действия',
    'sublinks': 'вложенные ссылки',
    'name': 'наименование',
    'type': 'тип',
    'owner': 'владелец',
    'notes': 'заметки',
    'copy': 'копировать',
    'error': 'ошибка',
    'home': 'главная',
    'login': 'войти',
    'logout': 'выйти',
    'Profile': 'Профиль',
    'settings': 'настройки',
    'Settings': 'Настройки',
    'version': 'версий | версия | версии',
    'cache': 'кэш',
    'documentation': 'документация',
    'request': 'запрос',
    'repository': 'репозиторий',
    'app info': 'инфо',
    'status': 'статус',
    'List is empty': 'Список пуст',
    'child instances': 'вложенные объекты',
    'enter value': 'введите значение',
    'more info': 'подробнее',
    'search by': 'поиск по полю',
    'Search': 'Поиск',
    'Selected': 'Выбрано',
    'Nothing found!': 'Ничего не найдено!',
    'Main settings': 'Основные настройки',
    'Custom settings': 'Другие настройки',
    'Remove from list': 'Убрать из списка',
    'Purge': 'Удалить',
    'Select an action for this instances': 'Выберите действие для объектов',
    'Username not': 'Имя пользователя не',

    # fields
    'Id': 'Идентификатор',
    'ID': 'Идентификатор',

    # x-menu generic
    'system': 'Система',
    'users': 'Пользователи',
    'Is active': 'Активен',
    'Is staff': 'Модератор',

    # multiactions button
    'actions on': 'действия над',
    'on item': 'элементов | элементом | элементами | элементами',

    'key': 'ключ',
    'value': 'значение',
    'new owner': 'новый владелец',
    'detail': 'детали',
    'description': 'описание',
    'username': 'никнейм',
    'is active': 'активен',
    'user': 'пользователь',
    'yes': 'да',
    'no': 'нет',
    'is staff': 'админ',
    'first name': 'имя',
    'last name': 'фамилия',
    'password': 'пароль',
    'Password': 'Пароль',
    'Change password': 'Сменить пароль',
    'Change_password': 'сменить пароль',
    'generate password': 'сформировать пароль',
    'repeat password': 'повторить пароль',
    'Repeat password': 'Повторить пароль',
    'Old password': 'Старый пароль',
    'New password': 'Новый пароль',
    'Confirm new password': 'Подтвердить новый пароль',
    'kind': 'вид',
    'mode': 'режим',
    'options': 'опции',
    'information': 'информация',
    'date': 'дата',

    'file n selected': 'нет файлов | 1 файл выбран | {n} файла выбрано | {n} файлов выбрано',
    'image n selected': 'нет изображений | 1 изображение выбрано | {n} изображения выбрано | {n} изображений выбрано',

    # filters
    'Ordering': 'Порядок сортировки',
    'is_active': 'активен',
    'Username': 'Ник',
    'First name': 'Имя',
    'Last name': 'Фамилия',

    'Page matching current url was not found': 'Cтраница, соответствующая данному url, не была найдена',
    'Unknown error': 'Неизвестная ошибка',
    'Error': 'Ошибка',

    'We\'ve emailed you instructions for setting your password, if an account exists with the email you entered. You should receive them shortly.': 'Мы отправили вам письмо с инструкцией для сброса пароля.',
    'If you don\'t receive an email, please make sure you\'ve entered the address you registered with, and check your spam folder.': 'Если вы не получили письмо, пожалуйста, убедитесь, что введен верный адрес, и проверьте папку со спамом.',
    'The password reset link was invalid, possibly because it has already been used. Please request a new Password reset.': 'Неверная ссылка для восстановления пароля, возможно она уже была использована. Пожалуйста, попробуйте заново.',
    'Please enter your new password twice so we can verify you typed it in correctly.': 'Пожалуйста, введите новый пароль.',
    'change my password': 'изменить пароль',
    'Your password has been set.  You may go ahead and log in now.': 'Ваш пароль был успешно установлен. Теперь вы можете войти.',
    'Please enter a correct %(username)s and password. Note that both fields may be case-sensitive.': 'Пожалуйста, введите корректные %(username)s и пароль. Внимание, поля могуь быть регистрозависимыми.',
    'or': 'или',
    'two-factor authentication code': 'код двухфакторной авторизации',
    'verify code': 'подтвердить код',
    'Enter the code from the two-factor app on your mobile device. If you\'ve lost your device, you may enter one of your recovery codes.': 'Введите код из двухфакторного приложения на мобильном устройстве. Если вы потеряли свое устройство, вы можете ввести один из кодов восстановления.',
    'Scan the image with the two-factor authentication app on your phone. If you can’t use a barcode, enter this text code instead.': 'Отсканируйте изображение с помощью приложения для двухфакторной аутентификации на вашем телефоне. Если вы не можете использовать штрих-код, введите вместо него текстовый код.',
    'Enter the six-digit code from the application': 'Введите шестисимвольный код из приложения',
    'code': 'код',
    'recovery codes': 'коды восстановления',
    'Recovery codes are used to access your account in the event you cannot receive two-factor authentication codes.': 'Коды восстановления используются для доступа к вашей учетной записи в случае, если вы не можете получить коды двухфакторной аутентификации.',
    'Disabling Two-Factor Authentication (2FA) will decrease your account\'s security': 'Отключение двухфакторной авторизации снизит безопасность вашей учетной записи.',
    'Enable': 'Включить',
    'Disable': 'Выключить',
    'Two factor authentication': 'Двухфакторная авторизация',
    'No {0} matches the given query.': "Ни один {0} не соответствует данному запросу.",
    'Password is not correct.': 'Неверный пароль.',
    'New passwords values are not equal.': 'Значения нового пароля не эквивалентны.',
    'This password is entirely numeric.': 'Этот пароль полностью числовой.',
    'Passwords do not match.': 'Пароли не совпадают.',
    "The two password fields didn't match.": 'Два поля пароля не совпадают.',
    'Dark theme': 'Темная тема',
    'Language': 'Язык',
    'height': 'высота',
    'width': 'ширина',
    'More info': 'Подробнее',
    'Dark mode': 'Темная тема',
    'Execute': 'Выполнить',
    'Domain should be setuped or username should be with @domain.name': "Домен должен быть настроен или логин должен содержать в конце @имя.домена",
    'Invalid name': 'Неправильное имя',
    'Should be [full.domain.name]': 'Должно быть в формате [полное.доменное.имя].',
    'Invalid registration email send.': 'Отправлена неверная регистрационная почта.',
    'To continue, need to accept the terms agreement.': 'Чтобы продолжить, необходимо принять условия соглашения.',

    # Проверить
    'Changes in settings are successfully saved. Please refresh the page.': 'Изменения в настройках успешно сохранены. Пожалуйста, обновите страницу.',
    'now': 'сейчас',
    'later': 'позже',
    'I accept the ': 'Я принимаю ',
    'Zoom': 'Зум',
    'Scale': 'Масштаб',
    'This file format is not supported': 'Данный формат файла не поддерживается',
    'Oops! Something went wrong.': 'Упс! Что-то пошло не так',
    '[Object not found]': '[Объект не найден]',
    'Execute action on {0}': 'Выполнить действие для {0}',
    '{n} instance': '0 объектов | {n} объекта | {n} объектов',
    'Open image': 'Открыть изображение',
    'Confirm action': 'Подтвердить действие',
    'Confirm': 'Подтвердить',
    'Cancel': 'Отменить',

    # field's validation
    # empty
    'Field "<b>{0}</b>" is empty.': 'Поле "<b>{0}</b>" пустое.',
    # required
    'Field "<b>{0}</b>" is required.': 'Поле "<b>{0}</b>" обязательно для заполнения.',
    # minLength
    'Field "<b>{0}</b>" is too short.<br> Field length should not be shorter, then {1}.': 'Длина поля "<b>{0}</b>" слишком мала.<br>Длина поля должна быть не меньше, чем {1}.',
    # maxLength
    'Field "<b>{0}</b>" is too long. <br> Field length should not be longer, then {1}.': 'Длина поля "<b>{0}</b>" слишком велика.<br>Длина поля должна быть не больше, чем {1}.',
    # min
    'Field "<b>{0}</b>" is too small.<br> Field should not be smaller, then {1}.': 'Значение поля "<b>{0}</b>" слишком мало.<br> Значение поля должно быть не меньше, чем {1}.',
    # max
    'Field "<b>{0}</b>" is too big.<br> Field should not be bigger, then {1}.': 'Значение поля "<b>{0}</b>" слишком велико.<br> Значение поля должно быть не больше, чем {1}.',
    # invalid
    '<b>{0} </b> value is not valid for <b>{1}</b> field.': 'Значение <b>{0}</b> не допустимо для поля <b>{1}</b>.',
    # email
    '<b>"{0}"</b> field should be written in <b>"example@mail.com"</b> format.': 'Поле <b>"{0}"</b> должно быть заполнено в формате <b>"example@mail.com"</b>.',

    # validators
    'imageValidationResolutionError': 'Должно быть не меньше {min} и не больше {max} px.',

    # instance operation success
    # add
    'Child "<b>{0}</b>" instance was successfully added to parent list.': 'Дочерний объект "<b>{0}</b>" был успешно добавлен в родительский список.',
    # create
    'New "<b>{0}</b>" instance was successfully created.': 'Новый объект "<b>{0}</b>" был успешно создан.',
    # remove
    '"<b>{0}</b>" {1} was successfully removed.': '"<b>{0}</b>" {1} был(а) успешно удален.',
    # save
    'Changes in "<b>{0}</b>" {1} were successfully saved.': 'Изменения в объекте {1} "<b>{0}</b>" были успешно сохранены.',
    # execute
    'Action "<b>{0}</b>" was successfully executed on "<b>{1}</b>" instance.': 'Действие "<b>{0}</b>" было успешно запущено на объекте "<b>{1}</b>".',
    # instance operation error
    # add:
    'An error occurred during adding of child "<b>{0}</b>" instance to parent list.<br> Error details: {1}': 'Во время добавления дочернего объекта "<b>{0}</b>" к родительскому списку произошла ошибка.<br> Подробнее: {1}',
    # create:
    'An error occurred during creation.<br>Error details:<br>{0}': 'Во время создания нового объекта произошла ошибка.<br>Подробнее:<br>{0}',
    # remove:
    'An error occurred during removal process of "<b>{0}</b>" {1}.<br> Error details: {2}': 'Во время удаления {1} "<b>{0}</b>" произошла ошибка.<br> Подробнее: {2}',
    # save:
    'An error occurred during process.<br>Error details:<br>{0}': 'Во время сохранения произошла ошибка.<br>Подробнее:<br>{0}',
    # execute:
    'An error occurred during <b>{0}</b>. Error details:<br>{1}': 'Во время запуска действия <b>{0}</b> произошла ошибка. Подробнее:<br>{1}',
    'Link': 'Ссылка',
    # csv
    'Actions': 'Действия',
    'Index': 'Индекс',
    'Download with headers': 'Скачать с заголовками столбцов',
}

SERVER_TRANSLATION = {
    # field's validation
    "value is not a valid list": "значение не является допустимым списком",
    "value is not a valid JSON": "значение не является допустимым JSON",
    "key {missing_key} is missing": "ключ {missing_key} отсутствует",
    "invalid key {invalid_key}": "неверный ключ {invalid_key}",

    # field help_text
    'A confirmation will be sent to your e-mail': 'На эту почту будет отправлено письмо для подтверждения регистрации',
    'Required. Inform a valid email address': 'Напишите вашу электронную почту',
    # validators
    'unsupported image file format, expected ({0}), got {1}': 'Неподдерживаемый формат изображения. Поддерживается ({0}), получен {1}',
    'For some reason, this image file cannot be opened': 'Не удалось открыть изображение',
    'Invalid image {0}. Expected from {1} to {2}, got {3}': 'Некорректная {0} изображения. Ожидалось от {1} до {2}, получено {3}',
    'This field must contain only digits but input: ': 'Это поле должно содержать только цифры, но введено: ',
    'Unsupported image file format': 'Не поддерживаемый: формат файла',
    'is not in listed supported types': 'не представлен в списке поддерживаемых типов',
    'Invalid image size orientations': 'Неправильные размеры изображения',
    'Current image size': 'Текущий размер изображения',

    # agreements
    'personal data processing policy': 'Политика обработки персональных данных',
    'I agree with ': 'Я согласен с ',
    'the personal data processing policy': 'политикой обработки персональных данных',
    'To continue, need to agree to the personal data processing policy.': 'Чтобы продолжить, необходимо согласиться '
                                                                          'с политикой обработки персональных данных.',
    'terms of agreement': 'лицензионное соглашение',

    # auth/reg
    'sign in to start your session': 'Войдите, чтобы начать сеанс',
    'confirm your email before logging in': 'подтвердите свою электронную почту перед входом',
    'Confirmation link is invalid or expired': 'Ссылка для подтверждения неверна или устарела',
    'sign in': 'войти',
    'sign up': 'зарегистрируйтесь',
    'forgot password': 'забыли пароль',
    'confirm password': 'подтвердите пароль',
    'register': 'регистрация',
    'fields with * is required': 'поля отмеченые * обязательны для заполнения',
    'Forgot your password? Enter your email address below, and an email with instructions for setting a new one will be sent.': 'Забыли пароль? Введите свой адрес, и мы отправим вам инструкцию для восстановления.',
    'reset my Password': 'сбросить пароль',
    'Invalid authentication code': 'Неверный код авторизации',
    'Secret string must be provided': 'Секретная строка должна быть заполнена',

    # filters help text
    'A unique integer value (or comma separated list) identifying this instance.': 'Уникальное числовое значение (или их последовательность разделенная запятой) идентифицирующая данный объект.',
    'A name string value (or comma separated list) of instance.': 'Имя объекта - строковое значение (или их последовательность разделенная запятой).',
    'Which field to use when ordering the results.': 'Поле, по которому производить сортировку результатов.',
    "Search by {}'s primary key or {}": 'Искать по ключу {} или {}',
    '_name': 'наименование',
    'Boolean value meaning status of user.': 'Значение, обозначающее статус пользователя.',
    'Users first name.': 'Имя пользователя',
    'Users last name.': 'Фамилия пользователя',
    'Users e-mail value.': 'Email пользователя',
}
