{% extends "registration/base.html" %}

{% block content %}
    <p style="color: #000000; margin-bottom: 25px">{% translate_text "Your password has been set.  You may go ahead and log in now." %}</p>
    <button class="btn btn-primary"><a style="color: #ffffff" href="{{ login_url }}">{% translate_text "sign in" capfirst %}</a></button>
{% endblock %}